'use strict';

let setup = (() => {
  var _ref = _asyncToGenerator(function* () {
    const ui = new _inquirer2.default.ui.BottomBar();

    const answers = yield _inquirer2.default.prompt([{
      name: 'kms.location',
      message: 'Enter the location for the keyring used to encrypt secrets.',
      default: DEFAULT_KMS_LOCATION,
      validate: function (val) {
        return KMS_LOCATIONS.includes(val) || 'Invalid KMS location.';
      },
      when: !_commander2.default.defaults
    }, {
      name: 'kms.keyring',
      message: 'Enter the name of the keyring used to encrypt secrets. ' + 'Will be created if it does not exist.',
      default: DEFAULT_KMS_KEYRING,
      when: !_commander2.default.defaults
    }, {
      name: 'kms.key',
      message: 'Enter the name of the key used to encrypt secrets. ' + 'Will be created if it does not exist.',
      default: DEFAULT_KMS_KEY,
      when: !_commander2.default.defaults
    }, {
      name: 'repo.name',
      message: 'Enter the name of the GitHub repository.',
      validate: function (val) {
        return !!val || 'GitHub repository name is required.';
      }
    }, {
      name: 'repo.token',
      message: 'Paste-in a GitHub personal access token with repo and hook permissions ' + '- generate at \nhttps://github.com/settings/tokens/new?scopes=repo,admin:repo_hook&description=cloudbuild\n',
      validate: function (val) {
        return !!val || 'Access token is required.';
      }
    }]);

    const {
      location = DEFAULT_KMS_LOCATION,
      keyring = DEFAULT_KMS_KEYRING,
      key = DEFAULT_KMS_KEY
    } = answers.kms || {};

    const webhookSecret = _crypto2.default.randomBytes(10).toString('hex');
    const projectId = yield _gcloud.googleApis.getProjectId();
    if (!projectId) {
      ui.log.write('Could not determine project id, try setting the GCLOUD_PROJECT environment variable');
      return;
    }

    ui.log.write('Setting up keys.');
    try {
      yield _gcloud.googleApis.createKeyring(location, keyring);
    } catch (err) {
      if (err.errors[0].reason !== 'alreadyExists') {
        throw err;
      }
    }
    try {
      yield _gcloud.googleApis.createKey(location, keyring, key);
    } catch (err) {
      if (err.errors[0].reason !== 'alreadyExists') {
        throw err;
      }
    }
    const projectNumber = yield _gcloud.googleApis.getProjectNumber();
    yield _gcloud.googleApis.setDecrypters(location, keyring, [`${projectId}@appspot.gserviceaccount.com`, `${projectNumber}@cloudbuild.gserviceaccount.com`]);

    ui.log.write('Encrypting secrets.');
    const encryptedWebhookSecret = yield _gcloud.googleApis.encryptKey(location, keyring, key, webhookSecret);
    const encryptedGithubToken = yield _gcloud.googleApis.encryptKey(location, keyring, key, answers.repo.token);

    const config = {
      kms: {
        location,
        keyring,
        key
      },
      encryptedWebhookSecret,
      repos: {
        [answers.repo.name]: {
          encryptedGithubToken,
          cloudbuild: {
            steps: [{
              id: 'fetch-source',
              // Use gcloud image instead of git, since the latter prevents GIT_ASKPASS from working.
              name: 'gcr.io/cloud-builders/gcloud',
              entrypoint: 'bash',
              args: ['-c', fetchWithMergeScript],
              env: ['GIT_ASKPASS=/tmp/cloudbuild-github-pass.sh'],
              secretEnv: ['GITHUB_TOKEN']
            }, {
              id: 'build-all',
              name: 'gcr.io/$PROJECT_ID/java-cloud-builder',
              entrypoint: './gradlew',
              args: ['continuousTest', '--stacktrace', '--no-daemon'],
              env: ['CI=true']
            }],
            timeout: '60m',
            secrets: [{
              kmsKeyName: `projects/${projectId}/locations/${location}/keyRings/${keyring}/cryptoKeys/${key}`,
              secretEnv: {
                GITHUB_TOKEN: encryptedGithubToken
              }
            }]
          }
        }
      }
    };
    const configYaml = _jsYaml2.default.safeDump(config, {
      // It's easier to manage build scripts without worrying about line length.
      lineWidth: 100000
    });

    ui.log.write('Outputting config.yml and index.js.');
    _fs2.default.writeFileSync('config.yml', configYaml);
    _fs2.default.writeFileSync('index.js', indexJs);

    ui.log.write('Done! Edit config.yml with any more customizations and run deploy.');
  });

  return function setup() {
    return _ref.apply(this, arguments);
  };
})();

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _process = require('process');

var _process2 = _interopRequireDefault(_process);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _jsYaml = require('js-yaml');

var _jsYaml2 = _interopRequireDefault(_jsYaml);

var _package = require('../../package.json');

var _package2 = _interopRequireDefault(_package);

var _gcloud = require('../gcloud');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * MIT License
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * Copyright (c) 2017 Choko (choko@curioswitch.org)
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * Permission is hereby granted, free of charge, to any person obtaining a copy
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * of this software and associated documentation files (the "Software"), to deal
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * in the Software without restriction, including without limitation the rights
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * copies of the Software, and to permit persons to whom the Software is
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * furnished to do so, subject to the following conditions:
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * The above copyright notice and this permission notice shall be included in all
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * copies or substantial portions of the Software.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * SOFTWARE.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            */

const KMS_LOCATIONS = ['global', 'asia-east1', 'asia-southeast1', 'europe-west1', 'us-central1', 'us-east1', 'us-west1'];

const fetchWithMergeScript = `echo 'echo $$GITHUB_TOKEN' > /tmp/cloudbuild-github-pass.sh && \\
chmod +x /tmp/cloudbuild-github-pass.sh && \\
git init && \\
git fetch --no-tags --progress $_REPOSITORY_URL +$_OTHER_BRANCH_REMOTE_REF:$_OTHER_BRANCH_LOCAL_REF +$_BASE_BRANCH_REMOTE_REF:$_BASE_BRANCH_LOCAL_REF && \\
git config remote.origin.url $_REPOSITORY_URL && \\
git config user.name cloudbuild && \\
git config user.email cloudbuild@example.com && \\
git checkout -f $_OTHER_BRANCH_LOCAL_REF && \\
git merge --no-ff --no-edit --no-progress $_BASE_BRANCH_LOCAL_REF && \\
MERGE_REV=$(git rev-parse HEAD^{commit}) && \\
git checkout -f $_BASE_BRANCH_LOCAL_REF && \\
git merge --no-ff --no-edit --no-progress $$MERGE_REV && \\
MERGE_REV=$(git rev-parse HEAD^{commit}) && \\
git checkout -f $$MERGE_REV
`;

const indexJs = `/*
 * MIT License
 *
 * Copyright (c) 2017 Choko (choko@curioswitch.org)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

module.exports = require('@curiostack/cloudbuild-github');
`;

const DEFAULT_KMS_LOCATION = 'us-central1';
const DEFAULT_KMS_KEYRING = 'cloudbuild';
const DEFAULT_KMS_KEY = 'github';

_commander2.default.version(_package2.default.version).option('--defaults', 'Use defaults').parse(_process2.default.argv);

setup().then(() => _process2.default.exit(), err => {
  console.error('Unexpected error.', err);
  _process2.default.exit(1);
});